program city;

#include "str.h"

const
  kDebug = False;

type
  Str255 = string;
  HeadingType = (N, NE, E, SE, S, SW, W, NW);
  CommandType = (HalfLeft, HalfRight, SharpLeft, SharpRight, Left, Right, Straight);

var
  avenue, street: Integer;
  heading: HeadingType;
  command: CommandType;
  goCount: Integer;
  startPosition: Str255;


function GetNumber (var line: Str255;
                var number: Integer): Boolean;

  begin
  line := line + '.';
  number := 0;
  while (line[1] in ['0'..'9']) do
    begin
    number := number * 10 + Ord(line[1]) - Ord('0');
    Delete(line, 1, 1);
    if (number >= 100) then
      begin
      number := 0;
      line := '*' + line;
      end;
    end;
  Delete(line, Length(line), 1);
  GetNumber := (number <> 0);
  end;


procedure Strip (var line: Str255);

  begin
  line := line + '.';
  while (line[1] = ' ') do
    Delete(line, 1, 1);
  Delete(line, Length(line), 1);

  line := '.' + line;
  while (line[Length(line)] = ' ') do
    Delete(line, Length(line), 1);
  Delete(line, 1, 1);

  while (Pos('  ', line) <> 0) do
    Delete(line, Pos('  ', line), 1);
  end;


procedure Position (line: Str255);

  var
    loop: Integer;
    mustSucceed: Boolean;
    id: Char;

  begin
  avenue := 0;
  street := 0;
  for loop := 1 to 2 do
    begin
    Strip(line);
    id := line[1];
    Delete(line, 1, 1);
    case id of
    'A': 
      begin
      mustSucceed := GetNumber(line, avenue);
      case line[1] of
      'E': 
        ;
      'W': 
        avenue := -avenue;
      otherwise
        Writeln('*** Avenues should be EW, but was [', line[1], '] in [', startPosition, ']');
      end;
      end;
    'S': 
      begin
      mustSucceed := GetNumber(line, street);
      case line[1] of
      'S': 
        street := -street;
      'N': 
        ;
      otherwise
        Writeln('*** Streets should be NS, but was [', line[1], '] in [', startPosition, ']');
      end;
      end;
    otherwise
      Writeln('*** Expected S or A, but got [', id, '] in [', startPosition, ']');
    end;
    if not mustsucceed then
      Writeln('*** Expected number on front of [', line, '] in [', startPosition, ']');
    Delete(line, 1, 1);
    end;
  Strip(line);
  if (line = 'NW') then
    heading := NW
  else if (line = 'SW') then
    heading := SW
  else if (line = 'NE') then
    heading := NE
  else if (line = 'SE') then
    heading := SE
  else if (Length(line) <> 1) then
    Writeln('*** Expected heading [', line, '] in [', startPosition, ']')
  else
    case line[1] of
    'N': 
      heading := N;
    'S': 
      heading := S;
    'E': 
      heading := E;
    'W': 
      heading := W;
    otherwise
      Writeln('Expected one of NSEW [', line, '] in [', startPosition, ']');
    end;
  end;


function Parse (line: Str255): Boolean;

  begin
  goCount := 1;
  Parse := True;
  Strip(line);
  if (Copy(line, 1, 5) = 'TURN ') then
    begin
    Delete(line, 1, 5);
    if (line = 'HALF LEFT') then
      command := HalfLeft
    else if (line = 'HALF RIGHT') then
      command := HalfRight
    else if (line = 'LEFT') then
      command := Left
    else if (line = 'RIGHT') then
      command := Right
    else if (line = 'SHARP LEFT') then
      command := SharpLeft
    else if (line = 'SHARP RIGHT') then
      command := SharpRight
    else
      Parse := False;
    end
  else if (Copy(line, 1, 3) = 'GO ') then
    begin
    command := Straight;
    Delete(line, 1, 3);
    if (Copy(line, 1, 9) = 'STRAIGHT ') then
      Delete(line, 1, 9);
    if GetNumber(line, goCount) then
      Parse := (line = '')
    else
      Parse := False;
    end
  else
    Parse := False;
  end;


procedure Step (sign, count: Integer;
                warn: Boolean);

  begin
  case heading of
  N: 
    street := street + sign * count;
  S: 
    street := street - sign * count;
  E: 
    avenue := avenue + sign * count;
  W: 
    avenue := avenue - sign * count;
  NE: 
    begin
    street := street + sign * count;
    avenue := avenue + sign * count;
    end;
  NW: 
    begin
    street := street + sign * count;
    avenue := avenue - sign * count;
    end;
  SE: 
    begin
    street := street - sign * count;
    avenue := avenue + sign * count;
    end;
  SW: 
    begin
    street := street - sign * count;
    avenue := avenue - sign * count;
    end;
  otherwise
    Writeln('Error in step');
  end;
{  if warn and not ((avenue in [-50..50]) and (street in [-50..50])) then
    Writeln('*** But you promised it would be inside city! [', startPosition, ']');}
  end;


procedure ChangeDirection (sign: Integer);

  var
    turn: Integer;

  begin
  case command of
  HalfLeft: 
    turn := -1;
  Left: 
    turn := -2;
  SharpLeft: 
    turn := -3;
  HalfRight: 
    turn := 1;
  Right: 
    turn := 2;
  SharpRight: 
    turn := 3;
  straight: 
    turn := 0;
  otherwise
    Writeln('Urk');
  end;
  heading := HeadingType((Ord(heading) + sign * turn + 8) mod 8);
  end;


function AtRoundabout: Boolean;
  begin
  AtRoundabout := (avenue mod 50 = 0) and (street mod 50 = 0);
  end;


function OnMainDrag: Boolean;
  begin
  OnMainDrag := ((avenue mod 50 = 0) and (heading in [N, S])) or ((street mod 50 = 0) and (heading in [E, W]));
  end;


function OnBoulevard: Boolean;
  begin
  OnBoulevard := ((avenue = street) and (heading in [NE, SW])) or ((street = -avenue) and (heading in [SE, NW]));
  end;


function OntoMainDrag: Boolean;
  begin
  ChangeDirection(1);
  OntoMainDrag := OnMainDrag;
  ChangeDirection(-1);
  end;


function OntoBoulevard: Boolean;
  begin
  ChangeDirection(1);
  OntoBoulevard := OnBoulevard;
  ChangeDirection(-1);
  end;


function OnMinor: Boolean;
  begin
  if ((avenue = street + 50) or (avenue = street - 50)) and (heading in [NE, SW]) then
    OnMinor := True
  else if ((-avenue = street + 50) or (-avenue = street - 50)) and (heading in [NW, SE]) then
    OnMinor := True
  else
    OnMinor := False;
  end;


function OnToMinor: Boolean;
  begin
  ChangeDirection(1);
  OnToMinor := OnMinor;
  ChangeDirection(-1);
  end;


function ValidCommand: Boolean;

  var
    valid: Boolean;

  begin
{Go to the intersection}
  Step(1, goCount, True);

{ Check would not go outside city if valid }
  ChangeDirection(1);
  Step(1, 1, True);
  Step(-1, 1, True);
  ChangeDirection(-1);


{ Check the turn direction }
  if AtRoundabout then
    valid := True
  else if OnMainDrag then
    valid := (command in [Left, Straight])
  else if OnBoulevard then
    valid := (command in [SharpLeft, Straight])
  else if OntoMainDrag then
    valid := (command = left)
  else if OntoBoulevard then
    valid := command = SharpLeft
  else if (avenue mod 25 = 0) and (street mod 25 = 0) then
    valid := True
  else if OnMinor then
    valid := not (command in [Left, Right])
  else if OntoMinor then
    valid := True
  else
    valid := command in [Left, Right, Straight];

  if valid then
    ChangeDirection(1)
  else
    Step(-1, goCount, True);

  ValidCommand := valid;
  end;


function ValidFinal: Boolean;

  begin
  ValidFinal := not (OnMainDrag or OnBoulevard);
  end;


procedure DoIt;

  var
    line: Str255;
    valid: Boolean;

  begin
  readln(line);
  if kDebug then Writeln('        ', line);
  while (line <> 'END') do
    begin
    startPosition := line;
    Position(line);

    if OnBoulevard or OnMainDrag then
      Writeln('*** Should not start on throughway [', startPosition, ']');
    Step(-1, 1, False);
    goCount := 1;
    command := Straight;
    if not ValidCommand then
      Writeln('*** Invalid starting direction [', startPosition, ']');

    readln(line);
    if kDebug then
      Writeln('        ', line);
    while (line <> 'STOP') do
      begin
      valid := Parse(line);
      if kDebug and not valid then
        Writeln('  Line failed to parse');
      if valid then
        begin
        valid := ValidCommand;
        if kDebug and not valid then
          Writeln('  Command direction was incorrect');
        end;

      readln(line);
      if kDebug then
        Writeln('        ', line);
      end;

    if not ValidFinal then
      begin writeln('Illegal stopping place') end
    else
      begin
      write('A', abs(avenue):1);
      if (avenue >= 0) then
        write('E')
      else
        write('W');
      write(' S', abs(street):1);
      if (street >= 0) then
        write('N')
      else
        write('S');
      write(' ');
      case heading of
      N: write('N');
      S: write('S');
      E: write('E');
      W: write('W');
      NE:write('NE');
      NW:write('NW');
      SE:write('SE');
      SW:write('SW');
      end;
      writeln
      end;

    if kDebug then
      readln;
    readln(line);
    if kDebug then
      writeln('        ', line);
    end;
  end;


begin
DoIt;
end.
